library(miceadds)
library(ggplot2)

rm(list=ls()) ; gc()

#Useful functions
pseudoF = function(X, k,wss){
  nk = length(k)
  n = nrow(X)
  T = sum(scale(X,scale=F)^2)
  W = rep(T, nk)
  for (i in 1:nk){
    set.seed(12345)
    #cli = kmeans(X, k[i],nstart=ns)
    W[i] = wss[i]
  }
  pF = ((T-W)/(k-1))/(W/(n-k))
  return(list(k=k, W=W, pF=pF))
}


gc()

setwd("/esarchive/scratch/molmo/")

for(dom in c(1,4,3)){
load(paste("input.data_D",dom,".Rdata",sep=""))
dim(input.data)

pseudo.F <- list()
input.data2 <- input.data

start = 1950 ; end =2022
calendario <- seq(as.Date(paste(as.character(start),"-01-01",sep="")),
                  as.Date(paste(as.character(end),"-12-31",sep="")),
                  by="day")

years <- as.character(seq(start,end,by=1))
pseudo.F <- list()

for(y in years){
  message(paste("Bootstrapping without ",y,sep=""))
  subsets <- which(format(calendario,"%Y")%in%years[-which(years==y)])
  input.data <- input.data2[subsets,]
  
  ward_clustering <- hclust(d = dist(input.data),method = "ward.D2")
  
  wss <- (nrow(input.data)-1)*sum(apply(input.data,2,var))
  for(x in 1:15){
    wss.sub <- NA
    for(j in 1:x){
      sub.data <- input.data[which(cutree(ward_clustering,k=x)==j),]
      wss.sub[j] <- (nrow(sub.data)-1)*sum(apply(sub.data,2,var))
    }
    wss[x] <- sum(wss.sub)
  }
  
  gc()
  pseudo.F[[y]] <- pseudoF(input.data,1:15,wss)$pF
  
  save(pseudo.F,file=paste("pseudo-F_listD",dom,"_new_years.Rdata",sep=""))
  gc()
}
}
# png(filename="pseudoF_bootsD2_years.png")
# plot(pseudo.F[[1]],type="l",main="Pseudo-F")
# for(i in 1:length(pseudo.F)){
#   lines(pseudo.F[[i]])
# }
# dev.off()

# png(filename="pseudoF_bootsD2_years.png")
# plot(pseudo.F[[1]],type="l",main="Pseudo-F")
# for(i in 1:length(pseudo.F)){
#   lines(pseudo.F[[i]])
# }
# dev.off()

data1 <- list()
for(dom in c(1,4,3)){
load(paste("pseudo-F_listD",dom,"_new_years.Rdata",sep=""))
pF <- matrix(unlist(pseudo.F),nrow = 15,ncol=length(pseudo.F))
pF[1,] <- NA

pF.mean <- rowMeans(pF,na.rm=T)
pF.sd <- apply(pF,1,"sd",na.rm=T)

data1[[dom]] <- data.frame(Mean=pF.mean,Low=pF.mean-pF.sd,High=pF.mean+pF.sd,Clusters=c(1:15),Classification=rep("D1",15))
}

data <- rbind(data[[1]],data[[4]],data[[3]])

p <- ggplot(data=data,aes(x=Clusters)) + geom_ribbon(aes(ymin=Low,ymax=High),alpha=0.4,fill="lightblue") + 
  theme_bw() + scale_x_continuous(breaks=c(1:15)) + ylab("Pseudo-F") + geom_line(aes(y=Mean)) + geom_point(aes(y=Mean))+
  facet_wrap(~Classification,nrow = 3,scales = "free")
ggsave(p,filename="pseudoF-bootstrapping_years.png",dpi=600,device="png",width=6,height=7)
